@@ -34,10 +34,10 @@ import itertools
 import os
 import imp
 import html.parser
+import hashlib
 
 from .util import TrackSubClasses
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -56,7 +56,7 @@ class FilterBase(object, metaclass=TrackSubClasses):
     @classmethod
     def filter_documentation(cls):
         result = []
-        for sc in list(cls.__subclasses__.values()):
+        for sc in TrackSubClasses.sorted_by_kind(cls):
             result.extend((
                 '  * %s - %s' % (sc.__kind__, sc.__doc__),
             ))
@@ -103,6 +103,7 @@ class AutoMatchFilter(FilterBase):
         logger.debug('Matching %r with %r result: %r', self, self.job, result)
         return result
 
+
 class RegexMatchFilter(FilterBase):
     """Same as AutoMatchFilter but matching is done with regexes"""
     MATCH = None
@@ -112,7 +113,11 @@ class RegexMatchFilter(FilterBase):
             return False
 
         d = self.job.to_dict()
-        result = all(v.match(d.get(k, None)) for k, v in self.MATCH.items())
+
+        # It's a match if we have at least one key/value pair that matches,
+        # and no key/value pairs that do not match
+        matches = [v.match(d[k]) for k, v in self.MATCH.items() if k in d]
+        result = len(matches) > 0 and all(matches)
         logger.debug('Matching %r with %r result: %r', self, self.job, result)
         return result
 
@@ -180,6 +185,7 @@ class GrepFilter(FilterBase):
         return '\n'.join(line for line in data.splitlines()
                          if re.search(subfilter, line) is not None)
 
+
 class InverseGrepFilter(FilterBase):
     """Filter which removes lines matching a regular expression"""
 
@@ -203,11 +209,11 @@ class StripFilter(FilterBase):
         return data.strip()
 
 
-class ElementByID(html.parser.HTMLParser):
-    def __init__(self, element_id):
+class ElementsByAttribute(html.parser.HTMLParser):
+    def __init__(self, name, value):
         super().__init__()
 
-        self._element_id = element_id
+        self._attributes = {name: value}
         self._result = []
         self._inside = False
         self._depth = 0
@@ -218,7 +224,7 @@ class ElementByID(html.parser.HTMLParser):
     def handle_starttag(self, tag, attrs):
         ad = dict(attrs)
 
-        if ad.get('id', None) == self._element_id:
+        if all(ad.get(k, None) == v for k, v in self._attributes.items()):
             self._inside = True
 
         if self._inside:
@@ -239,7 +245,7 @@ class ElementByID(html.parser.HTMLParser):
 
 
 class GetElementById(FilterBase):
-    """Get a HTML element by its ID"""
+    """Get an HTML element by its ID"""
 
     __kind__ = 'element-by-id'
 
@@ -247,6 +253,46 @@ class GetElementById(FilterBase):
         if subfilter is None:
             raise ValueError('Need an element ID for filtering')
 
-        element_by_id = ElementByID(subfilter)
+        element_by_id = ElementsByAttribute('id', subfilter)
         element_by_id.feed(data)
         return element_by_id.get_html()
+
+
+class GetElementByClass(FilterBase):
+    """Get all HTML elements by class"""
+
+    __kind__ = 'element-by-class'
+
+    def filter(self, data, subfilter=None):
+        if subfilter is None:
+            raise ValueError('Need an element class for filtering')
+
+        element_by_class = ElementsByAttribute('class', subfilter)
+        element_by_class.feed(data)
+        return element_by_class.get_html()
+
+
+class Sha1Filter(FilterBase):
+    """Calculate the SHA-1 checksum of the content"""
+
+    __kind__ = 'sha1sum'
+
+    def filter(self, data, subfilter=None):
+        self._no_subfilters(subfilter)
+        sha = hashlib.sha1()
+        sha.update(data.encode('utf-8', 'ignore'))
+        return sha.hexdigest()
+
+
+class HexdumpFilter(FilterBase):
+    """Convert binary data to hex dump format"""
+
+    __kind__ = 'hexdump'
+
+    def filter(self, data, subfilter=None):
+        self._no_subfilters(subfilter)
+        data = bytearray(data.encode('utf-8', 'ignore'))
+        blocks = [data[i * 16:(i + 1) * 16] for i in range(int((len(data) + (16 - 1)) / 16))]
+        return '\n'.join('%s  %s' % (' '.join('%02x' % c for c in block),
+                                     ''.join((chr(c) if (c > 31 and c < 127) else '.')
+                                             for c in block)) for block in blocks)
