@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of urlwatch (https://thp.io/2008/urlwatch/).
-# Copyright (c) 2008-2015 Thomas Perl <thp.io/about>
+# Copyright (c) 2008-2016 Thomas Perl <thp.io/about>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -31,7 +31,7 @@
 def ical2text(ical_string):
     import vobject
     result = []
-    if isinstance(ical_string, unicode):
+    if isinstance(ical_string, str):
         parsedCal = vobject.readOne(ical_string)
     else:
         try:
@@ -59,14 +59,3 @@ def ical2text(ical_string):
             result.append('%s: %s' % (date_str, event.summary.value))
 
     return '\n'.join(result)
-
-
-if __name__ == '__main__':
-    import sys
-
-    if len(sys.argv) == 2:
-        print ical2text(open(sys.argv[1]).read())
-    else:
-        print 'Usage: %s icalendarfile.ics' % (sys.argv[0])
-        sys.exit(1)
-
