@@ -28,22 +28,19 @@
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
-import subprocess
-import re
-
-import requests
-
 import email.utils
-import zlib
 import hashlib
-import base64
 import logging
-import ssl
-
+import os
+import re
+import subprocess
+import requests
 import urlwatch
+from requests.packages.urllib3.exceptions import InsecureRequestWarning
 
 from .util import TrackSubClasses
 
+requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
 
 logger = logging.getLogger(__name__)
 
@@ -58,10 +55,12 @@ class ShellError(Exception):
     def __str__(self):
         return '%s: Exit status %d' % (self.__class__.__name__, self.result)
 
+
 class NotModifiedError(Exception):
     """Exception raised on HTTP 304 responses"""
     ...
 
+
 class JobBase(object, metaclass=TrackSubClasses):
     __subclasses__ = {}
 
@@ -85,7 +84,7 @@ class JobBase(object, metaclass=TrackSubClasses):
     @classmethod
     def job_documentation(cls):
         result = []
-        for sc in list(cls.__subclasses__.values()):
+        for sc in TrackSubClasses.sorted_by_kind(cls):
             result.extend((
                 '  * %s - %s' % (sc.__kind__, sc.__doc__),
                 '    Required keys: %s' % (', '.join(sc.__required__),),
@@ -180,7 +179,7 @@ class UrlJob(Job):
     __kind__ = 'url'
 
     __required__ = ('url',)
-    __optional__ = ('cookies', 'data', 'method', 'ssl_no_verify')
+    __optional__ = ('cookies', 'data', 'method', 'ssl_no_verify', 'http_proxy', 'https_proxy')
 
     CHARSET_RE = re.compile('text/(html|plain); charset=([^;]*)')
 
@@ -192,6 +191,11 @@ class UrlJob(Job):
             'User-agent': urlwatch.__user_agent__,
         }
 
+        proxies = {
+            'http': os.getenv('HTTP_PROXY'),
+            'https': os.getenv('HTTPS_PROXY'),
+        }
+
         if job_state.timestamp is not None:
             headers['If-Modified-Since'] = email.utils.formatdate(job_state.timestamp)
 
@@ -201,10 +205,24 @@ class UrlJob(Job):
             self.method = "POST"
             logger.info('Sending POST request to %s', self.url)
 
-        response = requests.request(url=self.url, data=self.data,
-                                    headers=headers, method=self.method,
+        if self.http_proxy is not None:
+            proxies['http'] = self.http_proxy
+        if self.https_proxy is not None:
+            proxies['https'] = self.https_proxy
+
+        file_scheme = 'file://'
+        if self.url.startswith(file_scheme):
+            logger.info('Using local filesystem (%s URI scheme)', file_scheme)
+            return open(self.url[len(file_scheme):], 'rt').read()
+
+        response = requests.request(url=self.url,
+                                    data=self.data,
+                                    headers=headers,
+                                    method=self.method,
                                     verify=(not self.ssl_no_verify),
-                                    cookies=self.cookies)
+                                    cookies=self.cookies,
+                                    proxies=proxies)
+
         response.raise_for_status()
         if response.status_code == 304:
             raise NotModifiedError()
