@@ -27,27 +27,30 @@
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
+import difflib
+import email.utils
 import itertools
 import logging
-import difflib
-import time
-import email.utils
 import sys
+import time
 import cgi
 
-import urlwatch
+import requests
 
-from .util import TrackSubClasses
-from .mailer import Mailer
-from .mailer import SendmailMailer
+import urlwatch
 from .mailer import SMTPMailer
-import subprocess
+from .mailer import SendmailMailer
+from .util import TrackSubClasses
 
 try:
     import chump
 except ImportError:
     chump = None
 
+try:
+    from pushbullet import Pushbullet
+except ImportError:
+    Pushbullet = None
 
 logger = logging.getLogger(__name__)
 
@@ -72,7 +75,7 @@ class ReporterBase(object, metaclass=TrackSubClasses):
     @classmethod
     def reporter_documentation(cls):
         result = []
-        for sc in list(cls.__subclasses__.values()):
+        for sc in TrackSubClasses.sorted_by_kind(cls):
             result.extend((
                 '  * %s - %s' % (sc.__kind__, sc.__doc__),
             ))
@@ -81,12 +84,12 @@ class ReporterBase(object, metaclass=TrackSubClasses):
     @classmethod
     def submit_all(cls, report, job_states, duration):
         any_enabled = False
-        for name, cls in cls.__subclasses__.items():
+        for name, subclass in cls.__subclasses__.items():
             cfg = report.config['report'].get(name, {'enabled': False})
             if cfg['enabled']:
                 any_enabled = True
-                logger.info('Submitting with %s (%r)', name, cls)
-                cls(report, cfg, job_states, duration).submit()
+                logger.info('Submitting with %s (%r)', name, subclass)
+                subclass(report, cfg, job_states, duration).submit()
 
         if not any_enabled:
             logger.warn('No reporters enabled.')
@@ -148,12 +151,11 @@ class HtmlReporter(ReporterBase):
                 title = '<span title="{location}">{pretty_name}</span>'
             else:
                 title = '{location}'
-            title = '<h2><span class="verb">{verb}:</span> '+title+'</h2>'
+            title = '<h2><span class="verb">{verb}:</span> ' + title + '</h2>'
 
-            yield SafeHtml(title).format(
-                    verb=job_state.verb,
-                    location=job.get_location(),
-                    pretty_name=job.pretty_name())
+            yield SafeHtml(title).format(verb=job_state.verb,
+                                         location=job.get_location(),
+                                         pretty_name=job.pretty_name())
 
             content = self._format_content(job_state, cfg['diff'])
             if content is not None:
@@ -225,7 +227,7 @@ class TextReporter(ReporterBase):
             sep = line_length * '='
             yield from itertools.chain(
                 (sep,),
-                ('%02d. %s' % (idx+1, line) for idx, line in enumerate(summary)),
+                ('%02d. %s' % (idx + 1, line) for idx, line in enumerate(summary)),
                 (sep, ''),
             )
 
@@ -346,7 +348,7 @@ class EMailReporter(TextReporter):
         elif self.config['method'] == "sendmail":
             mailer = SendmailMailer(self.config['sendmail']['path'])
         else:
-            logger.error('Invalid entry for method {method}'.format(method = self.config['method']))
+            logger.error('Invalid entry for method {method}'.format(method=self.config['method']))
 
         # TODO set_password(options.email_smtp, options.email_from)
 
@@ -360,34 +362,107 @@ class EMailReporter(TextReporter):
         mailer.send(msg)
 
 
-class PushoverReport(TextReporter):
-    """Send summary via Pushover"""
+class WebServiceReporter(TextReporter):
+    MAX_LENGTH = 1024
 
-    __kind__ = 'pushover'
+    def web_service_get(self):
+        raise NotImplementedError
+
+    def web_service_submit(self, service, title, body):
+        raise NotImplementedError
 
     def submit(self):
-
         body_text = '\n'.join(super().submit())
 
         if not body_text:
-            logger.debug('Not sending pushover (no changes)')
+            logger.debug('Not sending %s (no changes)', self.__kind__)
             return
 
-        if len(body_text) > 1024:
-            body_text = body_text[0:1023]
+        if len(body_text) > self.MAX_LENGTH:
+            body_text = body_text[:self.MAX_LENGTH]
 
         try:
-            app = chump.Application(self.config['app'])
+            service = self.web_service_get()
         except:
-            logger.error("Failed to load chump - is it installed ('pip install chump')")
+            logger.error('Failed to load or connect to %s - are the dependencies installed and configured?',
+                         self.__kind__, exc_info=True)
             return
 
-        user = app.get_user(self.config['user'])
+        self.web_service_submit(service, 'Website Change Detected', body_text)
 
-        msg = user.create_message(
-            title='Website Change Detected',
-            message=body_text,
-            html=True,
-            sound='spacealarm')
 
+class PushoverReport(WebServiceReporter):
+    """Send summary via pushover.net"""
+
+    __kind__ = 'pushover'
+
+    def web_service_get(self):
+        app = chump.Application(self.config['app'])
+        return app.get_user(self.config['user'])
+
+    def web_service_submit(self, service, title, body):
+        msg = service.create_message(title=title, message=body, html=True, sound='spacealarm')
         msg.send()
+
+
+class PushbulletReport(WebServiceReporter):
+    """Send summary via pushbullet.com"""
+
+    __kind__ = 'pushbullet'
+
+    def web_service_get(self):
+        return Pushbullet(self.config['api_key'])
+
+    def web_service_submit(self, service, title, body):
+        service.push_note(title, body)
+
+
+class MailGunReporter(TextReporter):
+    """Custom email reporter that use mailgun service"""
+
+    __kind__ = 'mailgun'
+
+    def submit(self):
+        domain = self.config['domain']
+        api_key = self.config['api_key']
+        from_name = self.config['from_name']
+        from_mail = self.config['from_mail']
+        to = self.config['to']
+
+        filtered_job_states = list(self.report.get_filtered_job_states(self.job_states))
+        subject_args = {
+            'count': len(filtered_job_states),
+            'jobs': ', '.join(job_state.job.pretty_name() for job_state in filtered_job_states),
+        }
+        subject = self.config['subject'].format(**subject_args)
+
+        body_text = '\n'.join(super().submit())
+        body_html = '\n'.join(self.convert(HtmlReporter).submit())
+
+        if not body_text:
+            logger.debug('Not calling mailgun API (no changes)')
+            return
+
+        logger.debug("Sending mailgun request for domain:'{0}'".format(domain))
+        result = requests.post(
+            "https://api.mailgun.net/v3/{0}/messages".format(domain),
+            auth=("api", api_key),
+            data={"from": "{0} <{1}>".format(from_name, from_mail),
+                  "to": to,
+                  "subject": subject,
+                  "text": body_text,
+                  "html": body_html})
+
+        try:
+            json_res = result.json()
+
+            if (result.status_code == 200):
+                logger.info("Mailgun response: id '{0}'. {1}".format(json_res['id'], json_res['message']))
+            else:
+                logger.error("Mailgun error: {0}".format(json_res['message']))
+        except ValueError:
+            logger.error(
+                "Failed to parse Mailgun response. HTTP status code: {0}, content: {1}".format(result.status_code,
+                                                                                               result.content))
+
+        return result
